{****************************************************
Author:
	German Mendez Gonzalez 
	A Corunha - Galicia - Spain
	E-mail: gmendez@hotmail.com
	http://www.lander.es/~gmendez

TIconTaskBar: Delphi component that allow show an icon in the Win95 task bar.

This component and source are freeware.

Please don't remove my information from the source (Name, E-Mail, ...)
Thanks.
****************************************************}


unit IconTBar;

interface

uses
  controls,
  Windows,
  messages,
  classes,
  graphics,
  sysutils,
  forms,
  dialogs,
  ShellApi;

const
  IconOnID=1;
  IconOffID=0;

type
  TWinIconTaskBar = class(TWinControl)
  private
    { Private declarations }
    FOnDblClick,FOnClick,FOnEvent:TNotifyEvent;
    FOnMouseUp,FOnMouseDown:TMouseEvent;
    FOnMouseMove:TMouseMoveEvent;
    procedure Event(Var Message:TMessage); message WM_USER+1; //Recibe todos los mensajes
  protected
    { Protected declarations }
  public
    { Public declarations }
    property OnEvent: TNotifyEvent read FOnEvent write FOnEvent;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    property OnMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;

end;


type
  TIconTaskBar = class(TComponent)
  private
    { Private declarations }
    FIconOn,FIconOff:TIcon;
    IconOnID,IconOffID:Integer;
    FHint:String;
  protected
    { Protected declarations }
    FOnEvent,FOnShow,FOnClose,FOnDblClick,FOnClick:TNotifyEvent;
    FOnMouseUp, FOnMouseDown: TMouseEvent;
    FOnMouseMove: TMouseMoveEvent;
    FVisible:Boolean;
    procedure SetIconOn(Value: TIcon);
    procedure SetIconOff(Value: TIcon);
    property IconOn: TIcon read FIconOn write SetIconOn;
    property IconOff: TIcon read FIconOff write SetIconOff;
  public
    { Public declarations }
    FWinControl:TWinIconTaskBar;
    constructor Create(AOwner: TComponent); override;
    destructor destroy;override;
    procedure Close;
    procedure Show;
  published
    { Published declarations }
    property Hint: string read FHint write FHint;
    property Visible: Boolean read FVisible write FVisible;

    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;

    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    property OnMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;

    property OnEvent: TNotifyEvent read FOnEvent write FOnEvent;
  end;

procedure Register;

implementation


procedure TWinIconTaskBar.Event(Var Message:TMessage);
// Tipos Delphi:
// TMouseButton = (mbLeft, mbRight, mbMiddle);
// TShiftState = set of (ssShift, ssAlt, ssCtrl, ssLeft, ssRight, ssMiddle, ssDouble);
Var X,Y:Word;
    Shift:TShiftState;
begin
 if Assigned(FOnEvent) then FOnEvent(Self);
 shift:=[];
 X:=LOWORD(Message.lParam); // horizontal position of cursor
 Y:=HIWORD(Message.lParam); // vertical position of cursor

 case message.wparam of  // key flags
  MK_CONTROL:Shift:=Shift+[ssctrl];	//Set if the CTRL key is down.
  MK_MBUTTON:Shift:=Shift+[ssMiddle];	//Set if the middle mouse button is down.
  MK_RBUTTON:Shift:=Shift+[ssRight];	//Set if the right mouse button is down.
  MK_SHIFT:Shift:=Shift+[ssShift];	//Set if the SHIFT key is down.
 end;

 case Message.lParam of
//  BOTON IZQUIERDO
  WM_LBUTTONDBLCLK: begin
                     Shift:=Shift+[ssDouble];
                     if Assigned(FOnDblClick) then FOnDblClick(Self);
                    end;
  WM_LBUTTONUP: if Assigned(FOnClick) then FOnClick(Self);
  WM_LBUTTONDOWN: if Assigned(FOnMouseDown) then FOnMouseDown(Self,mbLeft,Shift,X,Y);

//  BOTON DERECHO
  WM_RBUTTONDBLCLK: begin
                     Shift:=Shift+[ssDouble];
                     if Assigned(FOnDblClick) then FOnDblClick(Self);
                    end;
  WM_RBUTTONUP: if Assigned(FOnClick) then FOnClick(Self);
  WM_RBUTTONDOWN: if Assigned(FOnMouseDown) then FOnMouseDown(Self,mbRight,Shift,X,Y);

//  BOTON MEDIO
  WM_MBUTTONDBLCLK: begin
                     Shift:=Shift+[ssDouble];
                     if Assigned(FOnDblClick) then FOnDblClick(Self);
                    end;
  WM_MBUTTONUP: if Assigned(FOnClick) then FOnClick(Self);
  WM_MBUTTONDOWN: if Assigned(FOnMouseDown) then FOnMouseDown(Self,mbMiddle,Shift,X,Y);

  WM_MOUSEMOVE: if Assigned(FOnMouseMove) then FOnMouseMove(Self,Shift,X,Y);
 end;

end;

constructor TIconTaskBar.Create(AOwner: TComponent);
begin
 inherited create(AOwner);

end;

procedure TIconTaskBar.Show;
var IconTaskBar:TNotifyIconData;
begin
 FWinControl:=TWinIconTaskBar.Create(Owner);
 FWinControl.Parent:=application.MainForm;//TForm(Owner);

  FWinControl.OnClick:= Self.OnClick;
  FWinControl.OnDblClick:= Self.OndblClick;
  FWinControl.OnMouseMove:= Self.OnMouseMove;
  FWinControl.OnMouseDown:= Self.OnMouseDown;
  FWinControl.OnEvent:= Self.OnEvent;

If IconOn=Nil then
begin
 FIconOn:=TIcon.Create;
 try
  FIconOn.Assign(Application.ICon);
 except
 end;
 IconOn.Assign(FIconOn);
end;

 With IconTaskBar do
 begin
  cbSize:=SizeOf(IconTaskBar);
  Wnd:=FWinControl.Handle;
  uID:=IconOnID;
  uFlags:= NIF_ICON Or NIF_MESSAGE Or NIF_TIP;
  uCallbackMessage:=WM_USER+1;
  hIcon:=IconOn.Handle;
  StrPCopy(szTip,FHint);
 end;
 Shell_NotifyIcon(NIM_ADD,@IconTaskBar);

 Visible := True;

 if Assigned(FOnShow) then FOnShow(Self);

end;

destructor TIconTaskBar.destroy;
begin
// FWinControl.Destroy;
 IconOn.Free;
 inherited destroy;
end;

procedure TIconTaskBar.Close;
var IconTaskBar:TNotifyIconData;
begin
 if Assigned(FOnClose) then FOnClose(Self);
 With IconTaskBar do
 begin
  cbSize:=SizeOf(IconTaskBar);
  Wnd:=FWinControl.Handle;
  uID:=IconOnID;
 end;
 Shell_NotifyIcon(NIM_DELETE,@IconTaskBar);
 Visible := False;
end;

procedure TIconTaskBar.SetIconOn(Value: TIcon);
begin
  IconOn.Assign(Value);
end;

procedure TIconTaskBar.SetIconOff(Value: TIcon);
begin
  IconOff.Assign(Value);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TIconTaskBar]);
end;

end.


